@@ -1,4 +1,4 @@
-// Copyright 2018-2020 Cargill Incorporated
+// Copyright 2018-2021 Cargill Incorporated
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -13,34 +13,111 @@
 // limitations under the License.
 
 use std::collections::{HashMap, HashSet};
+use std::convert::TryFrom;
 use std::path::Path;
 use std::time::Duration;
 
+use cylinder::VerifierFactory;
 #[cfg(feature = "service-arg-validation")]
 use splinter::service::validation::{ServiceArgValidationError, ServiceArgValidator};
-use splinter::{
-    orchestrator::{OrchestratableService, OrchestratableServiceFactory},
-    service::{FactoryCreateError, Service, ServiceFactory},
-    signing::SignatureVerifierFactory,
-};
+use splinter::service::{FactoryCreateError, Service, ServiceFactory};
 
 #[cfg(feature = "service-arg-validation")]
 use crate::hex::parse_hex;
 
-use super::{Scabbard, SERVICE_TYPE};
+use super::{Scabbard, ScabbardVersion, SERVICE_TYPE};
 
 const DEFAULT_STATE_DB_DIR: &str = "/var/lib/splinter";
 const DEFAULT_STATE_DB_SIZE: usize = 1 << 30; // 1024 ** 3
 const DEFAULT_RECEIPT_DB_DIR: &str = "/var/lib/splinter";
 const DEFAULT_RECEIPT_DB_SIZE: usize = 1 << 30; // 1024 ** 3
 
+#[cfg(feature = "factory-builder")]
+/// Builds new ScabbardFactory instances.
+#[derive(Default)]
+pub struct ScabbardFactoryBuilder {
+    state_db_dir: Option<String>,
+    state_db_size: Option<usize>,
+    receipt_db_dir: Option<String>,
+    receipt_db_size: Option<usize>,
+    signature_verifier_factory: Option<Box<dyn VerifierFactory>>,
+}
+
+#[cfg(feature = "factory-builder")]
+impl ScabbardFactoryBuilder {
+    /// Constructs a new builder.
+    pub fn new() -> Self {
+        Self::default()
+    }
+
+    /// Sets the state db directory to be used by the resulting factory.
+    pub fn with_state_db_dir(mut self, state_db_dir: String) -> Self {
+        self.state_db_dir = Some(state_db_dir);
+        self
+    }
+
+    /// Sets the state db size to be used by the resulting factory.
+    pub fn with_state_db_size(mut self, state_db_size: usize) -> Self {
+        self.state_db_size = Some(state_db_size);
+        self
+    }
+
+    /// Sets the receipt db directory to be used by the resulting factory.
+    pub fn with_receipt_db_dir(mut self, receipt_db_dir: String) -> Self {
+        self.receipt_db_dir = Some(receipt_db_dir);
+        self
+    }
+
+    /// Sets the receipt db size to be used by the resulting factory.
+    pub fn with_receipt_db_size(mut self, receipt_db_size: usize) -> Self {
+        self.receipt_db_size = Some(receipt_db_size);
+        self
+    }
+
+    /// Set the signature verifier factory to be used by the resulting factory.  This is a required
+    /// value, and omitting it will result in an [splinter::error::InvalidStateError] at build-time.
+    pub fn with_signature_verifier_factory(
+        mut self,
+        signature_verifier_factory: Box<dyn VerifierFactory>,
+    ) -> Self {
+        self.signature_verifier_factory = Some(signature_verifier_factory);
+        self
+    }
+
+    /// Build the final [ScabbardFactory] instance.
+    ///
+    /// # Errors
+    ///
+    /// Returns an InvalidStateError if a signature_verifier_factory has not been set.
+    pub fn build(self) -> Result<ScabbardFactory, splinter::error::InvalidStateError> {
+        let signature_verifier_factory = self.signature_verifier_factory.ok_or_else(|| {
+            splinter::error::InvalidStateError::with_message(
+                "A scabbard factory requires a signature verifier factory".into(),
+            )
+        })?;
+
+        Ok(ScabbardFactory {
+            service_types: vec![SERVICE_TYPE.into()],
+            state_db_dir: self
+                .state_db_dir
+                .unwrap_or_else(|| DEFAULT_STATE_DB_DIR.into()),
+            state_db_size: self.state_db_size.unwrap_or(DEFAULT_STATE_DB_SIZE),
+            receipt_db_dir: self
+                .receipt_db_dir
+                .unwrap_or_else(|| DEFAULT_RECEIPT_DB_DIR.into()),
+            receipt_db_size: self.receipt_db_size.unwrap_or(DEFAULT_RECEIPT_DB_SIZE),
+            signature_verifier_factory,
+        })
+    }
+}
+
 pub struct ScabbardFactory {
     service_types: Vec<String>,
     state_db_dir: String,
     state_db_size: usize,
     receipt_db_dir: String,
     receipt_db_size: usize,
-    signature_verifier_factory: Box<dyn SignatureVerifierFactory>,
+    signature_verifier_factory: Box<dyn VerifierFactory>,
 }
 
 impl ScabbardFactory {
@@ -49,7 +126,7 @@ impl ScabbardFactory {
         state_db_size: Option<usize>,
         receipt_db_dir: Option<String>,
         receipt_db_size: Option<usize>,
-        signature_verifier_factory: Box<dyn SignatureVerifierFactory>,
+        signature_verifier_factory: Box<dyn VerifierFactory>,
     ) -> Self {
         ScabbardFactory {
             service_types: vec![SERVICE_TYPE.into()],
@@ -79,15 +156,16 @@ impl ServiceArgValidator for ScabbardArgValidator {
         let admin_keys_str = args
             .get("admin_keys")
             .ok_or_else(|| ServiceArgValidationError("admin_keys argument not provided".into()))?;
-        let admin_keys: Vec<String> = serde_json::from_str(admin_keys_str).map_err(|err| {
+
+        let admin_keys = parse_list(admin_keys_str).map_err(|err| {
             ServiceArgValidationError(format!("failed to parse admin_keys list: {}", err,))
         })?;
 
         for key in admin_keys {
             let key_bytes = parse_hex(&key).map_err(|_| {
                 ServiceArgValidationError(format!(
-                    "{} is not a valid hex-formatted public key",
-                    key
+                    "{:?} is not a valid hex-formatted public key",
+                    key,
                 ))
             })?;
 
@@ -118,6 +196,7 @@ impl ServiceFactory for ScabbardFactory {
     /// - `coordinator_timeout`: the length of time (in milliseconds) that the network has to
     ///   commit a proposal before the coordinator rejects it (if not provided, default is 30
     ///   seconds)
+    /// - `version`: the protocol version for scabbard (possible values: "1", "2") (default: "1")
     fn create(
         &self,
         service_id: String,
@@ -125,9 +204,62 @@ impl ServiceFactory for ScabbardFactory {
         circuit_id: &str,
         args: HashMap<String, String>,
     ) -> Result<Box<dyn Service>, FactoryCreateError> {
-        Ok(Box::new(
-            self.create_scabbard(service_id, circuit_id, args)?,
-        ))
+        let peer_services_str = args.get("peer_services").ok_or_else(|| {
+            FactoryCreateError::InvalidArguments("peer_services argument not provided".into())
+        })?;
+
+        let peer_services = parse_list(peer_services_str)
+            .map_err(|err| {
+                FactoryCreateError::InvalidArguments(format!(
+                    "failed to parse peer_services list: {}",
+                    err,
+                ))
+            })?
+            .into_iter()
+            .collect::<HashSet<String>>();
+
+        let state_db_dir = Path::new(&self.state_db_dir);
+        let receipt_db_dir = Path::new(&self.receipt_db_dir);
+        let admin_keys_str = args.get("admin_keys").ok_or_else(|| {
+            FactoryCreateError::InvalidArguments("admin_keys argument not provided".into())
+        })?;
+
+        let admin_keys = parse_list(admin_keys_str).map_err(|err| {
+            FactoryCreateError::InvalidArguments(format!(
+                "failed to parse admin_keys list: {}",
+                err,
+            ))
+        })?;
+
+        let coordinator_timeout = args
+            .get("coordinator_timeout")
+            .map(|timeout| match timeout.parse::<u64>() {
+                Ok(timeout) => Ok(Duration::from_millis(timeout)),
+                Err(err) => Err(FactoryCreateError::InvalidArguments(format!(
+                    "invalid coordinator_timeout: {}",
+                    err
+                ))),
+            })
+            .transpose()?;
+        let version = ScabbardVersion::try_from(args.get("version").map(String::as_str))
+            .map_err(FactoryCreateError::InvalidArguments)?;
+
+        let service = Scabbard::new(
+            service_id,
+            circuit_id,
+            version,
+            peer_services,
+            &state_db_dir,
+            self.state_db_size,
+            &receipt_db_dir,
+            self.receipt_db_size,
+            self.signature_verifier_factory.new_verifier(),
+            admin_keys,
+            coordinator_timeout,
+        )
+        .map_err(|err| FactoryCreateError::CreationFailed(Box::new(err)))?;
+
+        Ok(Box::new(service))
     }
 
     #[cfg(feature = "rest-api")]
@@ -169,77 +301,16 @@ impl ServiceFactory for ScabbardFactory {
     }
 }
 
-impl OrchestratableServiceFactory for ScabbardFactory {
-    fn create_orchestratable_service(
-        &self,
-        service_id: String,
-        _service_type: &str,
-        circuit_id: &str,
-        args: HashMap<String, String>,
-    ) -> Result<Box<dyn OrchestratableService>, FactoryCreateError> {
-        Ok(Box::new(
-            self.create_scabbard(service_id, circuit_id, args)?,
-        ))
-    }
-}
-
-impl ScabbardFactory {
-    fn create_scabbard(
-        &self,
-        service_id: String,
-        circuit_id: &str,
-        args: HashMap<String, String>,
-    ) -> Result<Scabbard, FactoryCreateError> {
-        let peer_services_str = args.get("peer_services").ok_or_else(|| {
-            FactoryCreateError::InvalidArguments("peer_services argument not provided".into())
-        })?;
-        let peer_services = serde_json::from_str::<Vec<_>>(peer_services_str)
-            .map_err(|err| {
-                FactoryCreateError::InvalidArguments(format!(
-                    "failed to parse peer_services list: {}",
-                    err,
-                ))
-            })?
-            .into_iter()
-            .collect::<HashSet<String>>();
-        let state_db_dir = Path::new(&self.state_db_dir);
-        let receipt_db_dir = Path::new(&self.receipt_db_dir);
-        let admin_keys_str = args.get("admin_keys").ok_or_else(|| {
-            FactoryCreateError::InvalidArguments("admin_keys argument not provided".into())
-        })?;
-        let admin_keys = serde_json::from_str(admin_keys_str).map_err(|err| {
-            FactoryCreateError::InvalidArguments(format!(
-                "failed to parse admin_keys list: {}",
-                err,
-            ))
-        })?;
-
-        let coordinator_timeout = args
-            .get("coordinator_timeout")
-            .map(|timeout| match timeout.parse::<u64>() {
-                Ok(timeout) => Ok(Duration::from_millis(timeout)),
-                Err(err) => Err(FactoryCreateError::InvalidArguments(format!(
-                    "invalid coordinator_timeout: {}",
-                    err
-                ))),
-            })
-            .transpose()?;
-
-        let service = Scabbard::new(
-            service_id,
-            circuit_id,
-            peer_services,
-            state_db_dir,
-            self.state_db_size,
-            receipt_db_dir,
-            self.receipt_db_size,
-            self.signature_verifier_factory.create_verifier(),
-            admin_keys,
-            coordinator_timeout,
-        )
-        .map_err(|err| FactoryCreateError::CreationFailed(Box::new(err)))?;
-
-        Ok(service)
+/// Parse a service argument into a list. Check if the argument is in json or csv format
+/// and return the list of strings. An error is returned if json fmt cannot be parsed.
+fn parse_list(values_list: &str) -> Result<Vec<String>, String> {
+    if values_list.starts_with('[') {
+        serde_json::from_str(values_list).map_err(|err| err.to_string())
+    } else {
+        Ok(values_list
+            .split(',')
+            .map(String::from)
+            .collect::<Vec<String>>())
     }
 }
 
@@ -247,7 +318,7 @@ impl ScabbardFactory {
 mod tests {
     use super::*;
 
-    use splinter::signing::hash::HashVerifier;
+    use cylinder::secp256k1::Secp256k1Context;
 
     /// Verify that the scabbard factory produces a valid `Scabbard` instance.
     #[test]
@@ -267,6 +338,25 @@ mod tests {
         assert_eq!(&scabbard.circuit_id, "1");
     }
 
+    /// Verify that the scabbard factory produces a valid `Scabbard` instance if the service
+    /// arguments are commo seperated instead of json fmt.
+    #[test]
+    fn create_successful_no_json() {
+        let factory = get_factory();
+
+        let service = factory
+            .create("2".into(), "", "1", get_mock_args_no_json())
+            .expect("failed to create service");
+        assert_eq!(service.service_id(), "2");
+
+        let scabbard = (&*service)
+            .as_any()
+            .downcast_ref::<Scabbard>()
+            .expect("failed to downcast Service to Scabbard");
+        assert_eq!(&scabbard.service_id, "2");
+        assert_eq!(&scabbard.circuit_id, "1");
+    }
+
     /// Verify that the `coordinator_timeout` service argument is properly set for a new `Scabbard`
     /// instance.
     #[test]
@@ -318,7 +408,7 @@ mod tests {
             Some(1024 * 1024),
             Some("/tmp".into()),
             Some(1024 * 1024),
-            Box::new(HashVerifier),
+            Box::new(Secp256k1Context::new()),
         )
     }
 
@@ -336,4 +426,11 @@ mod tests {
         );
         args
     }
+
+    fn get_mock_args_no_json() -> HashMap<String, String> {
+        let mut args = HashMap::new();
+        args.insert("peer_services".into(), "0,1,3".into());
+        args.insert("admin_keys".into(), "".into());
+        args
+    }
 }
